option explicit

' Close All Tabs
' (C) 2014-2024 Leo Davidson
' 
' This is a script for Directory Opus.
' See http://www.gpsoft.com.au/DScripts/redirect.asp?page=scripts for development information.

' Called by Directory Opus to initialize the script
Function OnInit(initData)
	initData.name = "Close All Tabs (alt)"
	initData.desc = "Close all tabs, in all listers, which are in or below a specified path."
	initData.copyright = "(C) 2014-2024 Leo Davidson"
	initData.version = "1.2"
	initData.default_enable = true
	initData.min_version = "13.0.0"

	Dim cmd
	Set cmd = initData.AddCommand
	cmd.name = "CloseAllTabs2"
	cmd.method = "OnCloseAllTabs2"
	cmd.desc = "Close all tabs, in all listers, which are in or below a specified path."
	cmd.label = "Close All Tabs (alt)"
	cmd.template = "PATH"
	cmd.icon = "closetab"
End Function

' Implement the CloseAllTabs2 command
Function OnCloseAllTabs2(funcData)
	' The path argument is required, else we don't know which tabs to close.
	If (Not funcData.func.args.got_arg.path) Then
		OnCloseAllTabs = True ' Signifies failure
		Exit Function
	End If

	' We can't close listers or tabs while enumerating them, as it causes the enumeration to go wrong.
	' We also need to separately handle the case where the entire lister is to be closed, as closing the last tab
	' via Go TABCLOSE won't do anything.

	' So, we build up a Command object with a line to close each tab we want to close (only those from listers where at least one tab is not closed)
	' and we also build up a vector of Listers which we want to close entirely (those where every tab is to be closed, with none left).
	' After we finish enumerating tabs and listers, we run the command to close the tabs, then loop through our vector of Listers to close those as well.

	Dim anyTabs
	anyTabs = False

	Dim allTabsInLister

	Dim lister
	Dim tab

	Dim cmd
	Set cmd = DOpus.CreateCommand

	Dim tabsInLister
	Set tabsInLister = DOpus.NewVector

	Dim listersToClose
	Set listersToClose = DOpus.NewVector
	
	For Each lister in DOpus.listers

		allTabsInLister = True

		For Each tab in lister.tabs
			If (DOpus.FSUtil.ComparePath(tab.Path, funcData.func.args.path, "p")) Then
				' This tab needs to be closed.
				tabsInLister.push_back tab
			Else
				' At least one tab in this lister is not being closed.
				allTabsInLister = False
			End If
		Next

		If allTabsInLister Then
			' All tabs in the lister are being closed, so we will close the whole lister instead.
			listersToClose.push_back lister
		Else
			' Only some of the tabs in the lister are being closed, so add them to the command.
			For Each tab in tabsInLister
				anyTabs = True
				cmd.AddLine "Go TABCLOSE " & tab
			Next
		End If

		tabsInLister.clear
	Next

	If anyTabs Then
		' Run the command to close all the tabs (except ones where the whole lister will be closed).
		cmd.Run
		' Clear the Command object's lines, so we can re-use it.
		cmd.Clear
	End If

	For Each lister in listersToClose
		' Close each of the listers that need to be closed.
		cmd.SetSourceTab lister.ActiveTab
		cmd.RunCommand "Close"
	Next

End Function
